package nl.uvt.locator;

import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Nodes;
import nu.xom.xslt.XSLTransform;


public class MerescoConnection {

	List<LinkInfo> query( QueryData pQ ) throws LocatorException {
		List<LinkInfo> meResult = null;
		nu.xom.Document result;
		String url = Locator.locatorConfig.getProperty("merescoUri");
		String query = "";
		try {
			Properties prop = new Properties( Locator.sruDefaultProps );
			prop.setProperty("recordSchema","mods" );
			if ( pQ.genre.equals("book") || pQ.genre.equals("bookitem")){
				if ( pQ.isbn.length() > 0 ){
					Locator.logger.debug("ISBN: " + pQ.isbn );
					query = "mods.identifier=" + pQ.isbn;						
				}
				else if ( pQ.title.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.title + "\"";
				}
				else if ( pQ.atitle.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.atitle + "\"";
				}
				else
					// fails when no title or issn available
					return meResult;
			}
			else {
				if ( pQ.issn.length() > 0 )
					query = "mods.identifier=" + pQ.issn;
				else if ( pQ.title.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.title + "\"";
				}
				else if ( pQ.atitle.length() > 0 ) {
					query = "mods.titleInfo.title=\"" + pQ.atitle + "\"";
				}
				else
					// fails when no title or issn available
					return meResult;
			}
//			if ( pQ.year > 0 )
//				query += " and mods.originInfo.dateIssued=" + pQ.year;
//			if ( pQ.genre.equals("article") )
//				query += " and mods.genre=article"; 
//			else	if ( pQ.genre.equals("journal") )
//				query += " and mods.genre=series";
//			else	if ( pQ.genre.equals("preprint") )
//				query += " and mods.genre=preprint";
//			else
//				query += " and mods.genre=book"; 
			query += " and meta.repository.id exact opc-uvt-nl";
			prop.setProperty("query", query );
			url = Locator.createUrlString(url, prop );
			Locator.logger.debug("Try URL: " + url );
			result = Locator.httpGetDocument( url, Locator.httpConfig );
			int hits = Locator.extractHits( result );
			Locator.logger.debug("meresco numOfRecords=" + hits );
			if ( hits > 0 ){
//				Locator.logger.debug( "meresco doc:" + result.toXML());
				String sheetName = Locator.locatorConfig.getProperty("merescoSheet");
				meResult = getModsResults( result, pQ, sheetName );
			}
		}
		catch ( Exception e) {
			throw new LocatorException( "URL: '" + url + "' failed"); 
		}
		return meResult;
	}

		
		static public List<LinkInfo> getModsResults( Document doc, QueryData pQ, String sheetName ){
			List<LinkInfo> meResult = null;
			Builder builder = new Builder();
			try {
				MerescoInfo.modsContext.addNamespace("mods", MerescoInfo.modsNamespace);
				MerescoInfo.modsContext.addNamespace("xsi", MerescoInfo.schemaNamespace);
				MerescoInfo.modsContext.addNamespace("a", MerescoInfo.extNamespace);
				Locator.logger.debug( "use xslt sheet:" + sheetName );
				File f = Locator.getFile( sheetName );
				if ( f != null ){
					nu.xom.Document stylesheet = builder.build(f);
					XSLTransform transform = new XSLTransform(stylesheet);
					Nodes output = transform.transform( doc );
					nu.xom.Document standardDoc = XSLTransform.toDocument(output);
//					Locator.logger.debug( "transformed doc:" + standardDoc.toXML());
					Nodes records = standardDoc.query("//mods:mods", MerescoInfo.modsContext);
					if ( records.size() > 0 ) {
						meResult = new LinkedList<LinkInfo>();
						Locator.logger.debug("found " + records.size() + " real records");
						for ( int i=0; i < records.size(); ++i ){
							// every record may contain several occurrences
							// split these in separate records
							Locator.logger.debug(" try record: " + (i+1)  );
							MerescoInfo.fillMultipleRecord( records.get(i), meResult );
						}
					}
				}
				else {
					Locator.logger.fatal("unable to get a stylesheet:" + sheetName );
				}
			}
			catch( Exception e )
			{
				Locator.logger.error( "meresco exception", e );
			}
			return meResult;
		}


}
